Triggers = {}

function Triggers.idle()
    for projectile in Projectiles() do
        if is_fast_moving(projectile.type) then
            local x1, y1, z1 =
                ray_to_line_segment(
                projectile.x,
                projectile.y,
                projectile.z,
                math.rad(projectile.pitch),
                math.rad(projectile.yaw)
            )
            local t, x, y, z, polygon =
                projectile.polygon:check_collision(
                projectile.x,
                projectile.y,
                projectile.z,
                projectile.owner,
                x1,
                y1,
                z1,
                true,
                true
            )

            if x ~= nil and y ~= nil and z ~= nil and polygon ~= nil then
                projectile:position(x, y, z, polygon)
            end
        end
    end
end

function Triggers.projectile_created(projectile)
    if is_fast_moving(projectile.type) then
        local x1, y1, z1 =
            ray_to_line_segment(
            projectile.x,
            projectile.y,
            projectile.z,
            math.rad(projectile.pitch),
            math.rad(projectile.yaw)
        )
        local t, x, y, z, polygon =
            projectile.polygon:check_collision(
            projectile.x,
            projectile.y,
            projectile.z,
            projectile.owner,
            x1,
            y1,
            z1,
            true,
            true
        )

        if x ~= nil and y ~= nil and z ~= nil and polygon ~= nil then
            projectile:position(x, y, z, polygon)
        end
    end
end

function is_fast_moving(type)
    if
        type == "pistol bullet" or type == "rifle bullet" or type == "shotgun bullet" or type == "smg bullet" or
            type == "trooper bullet"
     then
        return true
    end

    return false
end

-- Adapted from Treellama code
function ray_to_line_segment(x0, y0, z0, pitch, yaw, reach)
    local max = 32767 / 1024
    local min = -32

    if not reach then
        reach = 110
    end

    local dx = math.cos(pitch) * math.cos(yaw)
    local dy = math.cos(pitch) * math.sin(yaw)
    local dz = math.sin(pitch)

    local x1 = x0 + reach * dx
    local y1 = y0 + reach * dy
    local z1 = z0 + reach * dz

    if x1 <= min then
        x1 = min
        y1 = y0 + dy * (min - x0) / dx
        z1 = z0 + dz * (min - x0) / dx
    end

    if x1 >= max then
        x1 = max
        y1 = y0 + dy * (max - x0) / dx
        z1 = z0 + dz * (max - x0) / dx
    end

    if y1 <= min then
        y1 = min
        x1 = x0 + dx * (min - y0) / dy
        z1 = z0 + dz * (min - y0) / dy
    end

    if y1 >= max then
        y1 = max
        x1 = x0 + dx * (max - y0) / dy
        z1 = z0 + dz * (max - y0) / dy
    end

    if z1 <= min then
        z1 = min
        x1 = x0 + dx * (min - z0) / dz
        y1 = y0 + dy * (min - z0) / dz
    end

    if z1 >= max then
        z1 = max
        x1 = x0 + dx * (min - z0) / dz
        y1 = y0 + dy * (min - z0) / dz
    end

    return x1, y1, z1
end
